#pragma once
// PlotXY window class, public declarations, C, not C++
#include <windows.h>

typedef enum{
 M_getCaps,M_size,M_precisX,M_precisY,
 M_getRangeX,M_getRangeY,M_get,M_set,
 M_insert,M_erase,M_lock,M_unlock,
 M_name,M_unit,
}XY_Msg;

typedef float XY_Float;	// data type commonly used for data

typedef struct{
 XY_Float x,y;
}XY_Point;

// This callback is invoked to get information about your trace data.
// Mostly called in reaction of WM_PAINT and some input events,
// so in the thread of the window.
typedef int (_stdcall*XY_cb) (LPARAM,XY_Msg,int,void*);

// In contrast to other controls, control structures are tighter packed.
// Not-to-specify things must be initialized with -1, even colors.
// (as -1 for color means full transparency otherwise.)
// Character data is WCHAR, as even Windows98 has working TextOutW() and related GDI functions
typedef struct{
 char itrace;		// 0 number of trace
 BYTE namebuflen;	// 1 only used for returning .name
 WORD flags;		// 2 editable, hidden, etc.
 char ixaxis,iyaxis;	// 4 want to connect to axis, or connected axis
 char iplotx,iploty;	// 6 want to occupy / create a plot area, or return assigned plot area
 char linewidth,	// 8 0 = 1 device pixel, else depending whether gdi or gdiplus
      linestyle,	// 9 0 = none, solid, dashed, dotted, dash-dotted
      linejoin,		// A 0 = linear, spline, jumpy: early, middle, late
      rop;		// B 0 = SRCCOPY, raster operation
 char fillstyle,fillto;	// C 0 = none, solid, //, \\, =, ||, some raster operation
 WCHAR marker;		// E 0 = none, One-character marker for data points
 COLORREF linecolor,	// 10 High-Byte = transparency value
	fillcolor,	// 14
	markercolor,	// 18
	minmaxcolor;	// 1C
 LPARAM lParam;		// 20 callback data
 XY_cb cb;		// 24/28 callback function pointer
 WCHAR*name;		// 28/30 no more than 64 characters are allowed
}XY_TraceInfo;		// 2C/38 (sizeof for 32 or 64 bit architecture)
// Other trace-related data comes from callback queries above.
// Especially data-related info like:
// * Whether data comes slowly (= history added by PlotXY) or fast (scope alike)
// * Whether data has multiple lines (virtual phospor scope alike) or range of mean/min/max/dev

enum{
 XY_FillTo_zero=-4,	// LabVIEW compatible codes?
 XY_FillTo_neg,
 XY_FillTo_pos,
 XY_FillTo_none,	// -1
// Nonnegative .fillto values will fill to trace number, IF in same plot area.
// Otherwise, no fill takes place.
// Note that fills are applied stacked from low to high trace number,
// so last-added traces will win. You can cope by
// * reordering traces
// * use ROP codes (GDI mode, best on black background)
// * use transparency (GDIplus mode)
// fill2color is used for filling min/max area.
};

// Window messages (to PlotXY window)
// It's similar to ComboBoxEx control messages CBEM_xxx
enum{
 XY_GETPLOTINFO=WM_USER,// lParam = XY_PlotInfo (information about entire PlotXY window or a specific plot area)
 XY_SETPLOTINFO,
 XY_ADDTRACE,		// wParam = unused, lParam = XY_TraceInfo*, inout itrace, ixaxis, iyaxis, iplotx, iploty
 XY_GETTRACE,		// wParam = unused, get information for that trace
 XY_SETTRACE,		// wParam = unused, possibly change itrace, ixaxis, iyaxis, iplotx, iploty
 XY_DELTRACE,		// wParam = iTrace
 XY_ADDAXIS,		// same for axes,   lParam = XY_AxisInfo*, inout flags, iaxis, iplotxy, col, row
 XY_GETAXIS,
 XY_SETAXIS,
 XY_DELAXIS,
 XY_ADDCURSOR,		// lParam = XY_CursorInfo*. Usually, only zero or one cursor is used
 XY_GETCURSOR,
 XY_SETCURSOR,
 XY_DELCURSOR,
 XY_ADDTICK,		// lParam = XY_TickInfo*.
 XY_GETTICK,		// Some of the elsewhere auto-generated ticks can be made static and named
 XY_SETTICK,		// The .iaxis member must always be valid. Ranges may be colored. (How?)
 XY_DELTICK,
// All these XY_ADDxx, XY_SETxx, and XY_DELxx messages will not update the screen.
// Moreover, trace data change doesn't update anything.
// You must send (from other thread better post) XY_UPDATE to make changes visible.
 XY_UPDATE,		// parameters to be defined!

/*** DEPRECATED MESSAGES ***/
// shorthand functions to plot data quickly (auto-trace generation, auto-axis assignment,
// NO plot area generation and assignment, NO digital data support, default line style and color
  XY_ADD_Y,		// add timed data to possibly multiple channels, uses current time stamp for X axis
  XY_ADD_TY,		// as above but first value is time from begin of measurement (should start with 0)
//	HIWORD(wParam) = first trace number
//	LOWORD(wParam) = trace count (i.e. array size)
//	lParam = pointer to const Float value(s)
  XY_SET_Y,		// replace one trace with new data, upto 65535 samples, X0 and delta-X are defaults
  XY_SET_XY,		// as above but value pairs, removes history data
//	HIWORD(wParam) = trace number
//	LOWORD(wParam) = sample count (i.e. array size)
//	lParam = pointer to const Float value(s)
  XY_SET_X0,		// Set X or T axis origin
  XY_SET_DX,		// Set X step of each data value (= remove per-value time stamp on history data)
//	HIWORD(wParam) = first trace number
//	LOWORD(wParam) = trace count (i.e. array size)
//	lParam = pointer to const double value(s)
};

