#pragma once
#include "plotxy.h"	// Klassenlose API
#include "range.h"	// Float
#include "../util/smartpointer.h"
#include "../util/vector.h"

class Axis;
class PlotXY;

/**********************************
 * Getter/Setter fr Daten-Arrays *
 **********************************/

struct DATA{
//  size_t size;	// Anzahl der Datenpunkte
//  Float precis;	// Genauigkeit (zur Festlegung des maximalen Zooms)
//  Range<Float> range;	// Der Datenanbieter MUSS den Bereich kennen!
//  const TCHAR*name,*unit;
//  virtual Float get(size_t) const=0;	// Den eigentlichen Datenzugriff muss der Aufrufer in einer abgeleiteten Klasse bereitstellen
 //virtual Float get3(unsigned,Float[3]) const=0;
//  Float operator[](size_t i) const {return get(i);}	// Operatorform
  enum ACTION{
//    query=0,	// Float wird ignoriert
    can_set=1,
//    offset=2,
    can_insert=4,
    can_append=8,
//    ins_after=8,
    can_erase=16,	// Float wird ignoriert
//    get1=32,	// Float ist Float* (Zeiger auf 1 Float)
//    get3=64,	// Float ist Float* (Zeiger auf 3 Float: Mittelwert, Minimum, Maximum)
  };
//  virtual ACTION change(size_t,Float*,ACTION) {return query;}
  virtual int getCaps(int=-1) const {return 0;}	// Was kann die Datenquelle?
  virtual int size() const =0;	// Anzahl der Datenpunkte
  virtual Float precis() const {return 0;}
  virtual bool getRange(Range<Float>&r) const {r.prepareFindRange(); for(int i=0; i<size(); i++) r.expand(get(i)); return true;}
  virtual Float get(int i) const =0;
//  virtual void set(int, Float) {}
//  virtual void insert(int,Float) {}
//  virtual void erase(int) {}
//  virtual Float*lock() {return 0;}
//  virtual void unlock() {}
  virtual const TCHAR*name() const {return 0;}
  virtual const TCHAR*unit() const {return 0;}
  virtual ~DATA() {}
};	// vielleicht besser nur _ein_ Funktionszeiger?? Mit t0, dt, precis?


/*
struct NamedObject{
 const TCHAR*name;
 NamedObject(const TCHAR*n):name(n) {}
 NamedObject() {}
};
*/
struct TRACEATTR{
 uniq_ptr<DATA>xydata[2];
 char icurve;	// -1
 char itrace;	// Nummer des Traces, -1 = anhngen
#pragma warning(disable:4201)	// nameless struct/union
 union{
  struct{
   char iplotx;	// Plotbereich-Spalte (nullbasiert), -1 = kein Wunsch -> je nach Achsen oder 0
   char iploty;	// Plotbereich-Zeile (nullbasiert),  -1 = kein Wunsch -> je nach Achsen oder 0
   char ixaxis;	// X-Achsennummer, -1 = nchstbeste oder neue Achse -> je nach Plotbereich
   char iyaxis;	// Y-Achsennummer, -1 = nchstbeste oder neue Achse -> je nach Plotbereich
  };		// Das Verhalten bei No-Match von Achsennummer, Seite oder Plotbereich ist undefiniert!
  struct{
   char iplotxy[2];
   char ixyaxis[2];
  };
  struct{
   Point<char>iplot,iaxis;
  };
 };
#pragma warning(default:4201)
 char linestyle;	// Strichelung
 char fillto;		// Flchenfll-Kode (siehe LabVIEW)
 char linewidth;	// Linienbreite
 Color tcolor,fcolor,mcolor;	// Farbe fr Trace,Polygon und Marker
 TCHAR marker;		// Punktmarkierung
 cstr<>name;
 DWORD flags;
 enum{
  JOINLINEAR=0x01,
  JOINLATE=0x02,
  JOINVP=0x10,	// Dicke der Linie variabel, umgekehrt proportional zur Lnge
  DIGITAL=0x40,
  VISIBLE=0x80,
  CHECK_XRANGE=0x100,	// Wenn sich X-Daten gendert haben, oder bei Erstellung von Trace
  CHECK_YRANGE=0x200,	// Wenn sich Y-Daten gendert haben, ...
  EDITABLE=0x10000,	// Was davon (x/y) hngt von xdata/ydata ab, gibt die Mglichkeiten vor u.a. fr Hittest
  bNeedUpdateRange=28,
  bNeedCalcPolys=30,
 };
 Font::Data fdmark;	// Schrift fr Marker
 // TODO: Trigger?
 TRACEATTR():icurve(-1),itrace(-1),iplotx(-1),iploty(-1),ixaxis(0),iyaxis(1),
	linestyle(0),fillto(0),linewidth(0),marker(0),flags(0) {}
};

class Trace:public TRACEATTR{
  friend class Axis;
  friend class PlotXY;
  PlotXY&pl;
  Axis&xaxis,&yaxis;
  Trace*tr_fillto;	// nur bei fillto>=0
  typedef vector<Point<> > points_t;
  points_t fill,mark;
  typedef vector<uniq_ptr<points_t> > lines_t;
  lines_t lines;
  Pen pline;
  Brush bfill,bline;	// bline bei JOINVP = Verbindung per Polygon
  Font fmark;		// Schriftart fr Marker-Symbole
  Trace(PlotXY&,TRACEATTR&);
  char measure(Margin<>&padding) const;
//  int textWidth(const TCHAR*,int) const;
  void paintPoly(HDC);
  void paintLine(HDC);
  void paintMark(HDC);
  void calcpolys();
  bool visible() const {return !!(flags&VISIBLE);}
  void setVisible(bool v) {if (v) flags|=VISIBLE; else flags&=~VISIBLE;}
  int name(TCHAR*,int) const;
  TCHAR*name() const;
  void name(TCHAR*,int);
  int defaultName(TCHAR*,int) const;
  Color penColor() const;
  void penColor(Color);
  Color penDefaultColor() const;
  int penWidth() const;
  void penWidth(int);
  int penDash() const;
  void penDash(int);
  int penJoin() const;
  void penJoin(int);
  int penJoinOptions(TCHAR*,int) const;
  unsigned penMarker() const;
  void penMarker(unsigned);
  Color fillColor() const;
  void fillColor(Color);
  int fillTo() const;
  void fillTo(int);
  int fillToOptions(TCHAR*,int) const;
  DATA*getData(bool y) const{return xydata[y].get();}
//  Axis*getAxis(bool y) const{return xyaxis[y];}
//  void setAxis(bool y, Axis*ax) {xyaxis[y]=ax;}
  Range<Float>cachedRanges[2];
  Range<Float>&getRange(size_t xysel);
  bool editable() const {return !!(flags&EDITABLE);}
  HMENU popup();
//  void handleMeasureItem(WPARAM,LPARAM);
//  void handleDrawItem(WPARAM,LPARAM);
  Rect<>rcPoly() const {return Rect<>();}
  Point<>Centroid() const {return Point<>();}
public:
  ~Trace();		// check arrange
};
