#pragma once
// An immutable string as in JavaScript, PHP etc.
// The object is simply a pointer to zero-terminated string.
#include <windows.h>	// TCHAR, lstrlen, lstrcpy, HeapXxx
#include "noexcept.h"	// swap, placement new

// wie array<> aber mit Nullterminierung und Zeiger in Heap
// Liefert nur const-Zeiger, der String ist immutable, also kein Stringbuffer
// Da hier Strings im Gegensatz zu Vektoren umherbewegt werden sollen,
// mssen Kopierkonstruktor und Zuweisungsoperator tiefe Kopien machen.
template<class T=TCHAR> class cstr{
 T*ptr;	// never NULL
public:
 cstr(const T*p=0):ptr(new T[(p?lstrlen(p):0)+1]) {if (p) lstrcpy(ptr,p); else *ptr=0;}
 cstr(const cstr<T>&o):ptr(new T[lstrlen(o.ptr)+1]) {lstrcpy(ptr,o.ptr);}	// Defaultkopierkonstruktor ersetzen
 ~cstr() {delete[]ptr;}
 cstr<T>&operator=(const T*p) {delete[]ptr; new(this)cstr(p); return*this;}
 cstr<T>&operator=(const cstr<T>&o) {delete[]ptr; new(this)cstr(o); return*this;}		// Defaultkopierzuweisung ersetzen
#if __cplusplus >= 201103L					// if C++11 or higher
 cstr(cstr<T>&&o):ptr(o.ptr) {o.ptr=0;}		// move constructor
 cstr<T>&operator=(cstr<T>&&o) {swap(ptr,o.ptr); return*this;}
#endif
#if __cplusplus >= 202002L					// if C++20 or higher
 auto operator<=>(const T*o) const {return lstrcmp(ptr,o)<=>0;}
#else
 bool operator==(const T*o) const {return lstrcmp(ptr,o)==0;}
 bool operator!=(const T*o) const {return lstrcmp(ptr,o)!=0;}
 bool operator< (const T*o) const {return lstrcmp(ptr,o)< 0;}
 bool operator<=(const T*o) const {return lstrcmp(ptr,o)<=0;}
 bool operator> (const T*o) const {return lstrcmp(ptr,o)> 0;}
 bool operator>=(const T*o) const {return lstrcmp(ptr,o)>=0;}
#endif
 operator const T*() const {return ptr;}	// liefert Stringzeiger
 const T*c_str() const {return ptr;}		// dasselbe fr printf-vararg
 int length() const {return lstrlen(ptr);}	// int ist unter Windows handlicher als size_t
 size_t size() const {return length();}
 bool empty() const {return !*ptr;}
 bool operator!() const {return empty();}
};

/* Besser:
 Leerer String = Nullpointer, c_str() liefert dann Zeiger auf Nullpointer
	= valider nullterminierter String
 Stringzeiger mit Lngen-TCHAR und Usage-TCHAR voran erspart viel strlen() und memcpy(),
 der Zeiger wird so zum shared_ptr. In Ermangelung der Move-Semantik von C++11.
 Gre des String-Objekts bleibt sizeof(TCHAR*).
 String-Objekt bleibt binrkompatibel zu const TCHAR*.
 Kann mit konstanten Objekten (usage==0) initialisiert werden.
 Kann Nullbytes enthalten.
 operator+ zum Verketten.
 find(), substr() und substring() wie in JavaScript.
 Allocator bleibt erst mal new[]TCHAR/delete[]TCHAR  oder weiterer Template-Parameter.
*/
